// $Id: IRC.h 1396 2005-09-17 00:12:12Z vern $

// An IRC analyzer contributed by Roland Gruber.

#ifndef irc_h
#define irc_h

#include "NVT.h"

/**
* \brief Main class for analyzing IRC traffic.
*/
class IRC_Conn : public TCP_Connection {
public:
	/**
	* \brief Constructor, builds a new analyzer object.
	*/
	IRC_Conn(NetSessions* s, HashKey* k, double t, const ConnID* id,
			const struct tcphdr* tp);

	/**
	* \brief Called when connection is closed.
	*/
	void Done();

	/**
	* \brief New input line in network stream.
	*
	* \param sender the origin of this line
	* \param length the line length
	* \param line pointer to line start
	*/
	void NewLine(TCP_ContentLine* sender, int length, const char* line);


protected:
	/**
	* \brief Sets the content processors.
	*/
	void BuildEndpoints();

	/** \brief used to see if data was processed, 0 unless NewLine was called */
	int did_deliver;
	TCP_NVT* o_nvt;
	TCP_NVT* r_nvt;

private:
	/** \brief counts number of invalid IRC messages */
	int invalid_msg_count;

	/** \brief maximum count of invalid IRC messages */
	int invalid_msg_max_count;

	/**
	* \brief Splits a string into its words which are separated by
	* the split character.
	*
	* \param input string which will be splitted
	* \param split character which separates the words
	* \return vector containing words
	*/
	vector<string> SplitWords(const string input, const char split);

};

#ifdef HAVE_LIBZ

#include "zlib.h"

/**
* \brief Decompresses input data on demand.
*/
class TCP_NVT_ZLIB : public TCP_NVT {
public:
	/**
	* \brief Constructor, just call parent constructor and initialize
	* z_stream.
	*/
	TCP_NVT_ZLIB(TCP_Endpoint* endp, int is_NUL_sensitive, int skip_partial,
			int CR_LF_as_EOL = LF_as_EOL);

	/**
	* \brief Decompresses given data if possible.
	*/
	void DoDeliver(int seq, int len, const u_char* data);

	~TCP_NVT_ZLIB();

protected:
	z_stream* zip;

private:
	enum GZIP_STATUS { NONE, ZIP_OK, ZIP_FAIL };

	/** \brief last compression status */
	GZIP_STATUS zip_status;
	/** \brief number of packets with current compression status */
	unsigned int zip_number;
	/** \brief true, if compression succeeded before */
	bool zip_ok_before;
	/** \brief true, if server-to-server connection */
	bool is_server_connection;

	/** \brief used to decompress server connections */
	void init_zStream();
};
#endif

#endif
