// $Id: pac_exception.h 1325 2005-09-07 17:33:57Z vern $

#ifndef pac_exception_h
#define pac_exception_h

#include <string>
using namespace std;

#include "pac.h"

class Exception
{
public:
	Exception(const Object* o, const char* msg = 0)
		{
		msg_ = o->Location();
		msg_ += ": error : ";
		if ( msg )
			msg_ += msg;
		if ( pac_debug ) 
			abort();
		}

	const char* msg() const 	{ return msg_.c_str(); }
	void append(const char* s) 	{ msg_ += s; }

private:
	string msg_;
};

class ExceptionIDNotFound : public Exception
{
public:
	ExceptionIDNotFound(const ID* arg_id)
		: Exception(arg_id), id(arg_id) 
		{
		append(fmt("`%s' undeclared", id->Name()));
		}
	const ID* getID() const { return id; }

protected:
	const ID* id;
};

class ExceptionIDRedefinition : public Exception
{
public:
	ExceptionIDRedefinition(const ID* arg_id)
		: Exception(arg_id), id(arg_id) 
		{
		append(fmt("`%s' redefined", id->Name()));
		}
	const ID* getID() const { return id; }

protected:
	const ID* id;
};

class ExceptionCyclicDependence : public Exception
{
public:
	ExceptionCyclicDependence(const ID* arg_id)
		: Exception(arg_id), id(arg_id) 
		{
		append(fmt("cyclic dependence through `%s'", 
			id->Name()));
		}
	const ID* getID() const { return id; }

protected:
	const ID* id;
};

class ExceptionPaddingError : public Exception
{
public:
	ExceptionPaddingError(const Object* o, const char* arg_msg)
		: Exception(o)
		{
		append(arg_msg);
		}
};

class ExceptionIDNotField : public Exception
{
public:
	ExceptionIDNotField(const ID* arg_id)
		: Exception(arg_id), id(arg_id) 
		{
		append(fmt("ID `%s' is not a field", 
			id->Name()));
		}
	const ID* getID() const { return id; }

protected:
	const ID* id;
};

class ExceptionNonConstExpr : public Exception
{
public:
	ExceptionNonConstExpr(const Expr* expr)
		: Exception(expr)
		{
		append(fmt("Expression `%s' is not constant", expr->Orig()));
		}

protected:
	const Expr *expr;
};

#endif /* pac_exception_h */
